# logging_config.py

import logging
import os
from datetime import datetime
from rich.logging import RichHandler

LOG_DIR = "logs"

def setup_logging():
    os.makedirs(LOG_DIR, exist_ok=True)

    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    # Clear handlers (prevents duplicates)
    if logger.hasHandlers():
        logger.handlers.clear()

    # ---------- Generate unique session filename ----------
    timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    logfile = os.path.join(LOG_DIR, f"session_{timestamp}.log")

    # ---------- Console handler (pretty) ----------
    console_handler = RichHandler(
        rich_tracebacks=True,
        markup=True
    )
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(logging.Formatter("%(message)s"))

    # ---------- File handler (detailed) ----------
    file_handler = logging.FileHandler(
        logfile,
        encoding="utf-8"
    )
    file_handler.setLevel(logging.DEBUG)

    file_handler.setFormatter(logging.Formatter(
        "%(asctime)s | %(name)s | %(levelname)-8s | "
        "%(filename)s:%(lineno)d | %(message)s"
    ))

    logger.addHandler(console_handler)
    logger.addHandler(file_handler)

    logger.propagate = False

    # Optional: log where the file is
    logger.info(f"Logging to {logfile}")



from logging_config import setup_logging
import logging
from converter import convert
import sys
import argparse


setup_logging()

logger = logging.getLogger(__name__)

logger.info("Programm started!")


from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QLineEdit, QMainWindow,
    QMenuBar, QPlainTextEdit, QPushButton, QSizePolicy,
    QStatusBar, QVBoxLayout, QWidget, QFileDialog)

#UI class genrated from layout.ui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1920, 1080)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lineEdit = QLineEdit(self.centralwidget)
        self.lineEdit.setObjectName(u"lineEdit")

        self.horizontalLayout.addWidget(self.lineEdit)

        self.filePickButton = QPushButton(self.centralwidget)
        self.filePickButton.setObjectName(u"filePickButton")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filePickButton.sizePolicy().hasHeightForWidth())
        self.filePickButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.filePickButton)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.pushButton = QPushButton(self.centralwidget)
        self.pushButton.setObjectName(u"pushButton")

        self.verticalLayout.addWidget(self.pushButton)

        self.textBox = QPlainTextEdit(self.centralwidget)
        self.textBox.setObjectName(u"textBox")
        self.textBox.setReadOnly(True)

        self.verticalLayout.addWidget(self.textBox)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 800, 30))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"AsciiImg", None))
        self.filePickButton.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.pushButton.setText(QCoreApplication.translate("MainWindow", u"Convert", None))
  

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        font = QFont("Courier New")   # or "Consolas"
        font.setStyleHint(QFont.Monospace)

        self.ui.textBox.setFont(font)

        self.ui.filePickButton.clicked.connect(self.select_file)
        self.ui.pushButton.clicked.connect(self.clicked_convert)

    def select_file(self):
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Pick File",
            "",
            "Images (*.png *.jpg *.jpeg)"
        )

        self.ui.lineEdit.setText(file_path)

    def clicked_convert(self):
        ascii_art = convert(self.ui.lineEdit.text())
        self.ui.textBox.setPlainText(ascii_art)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="AsciiImg tool")
    parser.add_argument("image", nargs="?", help="Path to image to convert")
    args = parser.parse_args()

    if not args.image:
        # Launch GUI
        app = QApplication(sys.argv)
        window = MainWindow()
        window.show()
        sys.exit(app.exec())
    else:
        # Run CLI mode
        ascii_art = convert(args.image)
        print(ascii_art)


import logging
from PIL import Image

ascii_symbols = [" ","-","•","+","=","#","@"]

logger = logging.getLogger(__name__)

def convert(img_path : str, width : int = 120):
    logger.info(f"Opening image '{img_path}...'")

    img = Image.open(img_path)
    width, height = img.size
    logger.info("Converting image to grayscale...")
    img = img.convert("L")

    new_width = 120
    aspect_ratio = height / width
    new_height = int(new_width * aspect_ratio * 0.55)

    logger.info(f"Resizing image to {new_width} x {new_height}...")

    img = img.resize((new_width, new_height))

    logger.info("Calculating brightness for each pixel and assigning symbol...")

    pixels = img.load()

    ascii_art = ""

    for y in range(new_height):
        for x in range(new_width):
            brightness = pixels[x, y]  # value 0–255
            level = brightness * len(ascii_symbols) // 256
            logger.debug(f"({x},{y}) has a brightness of {brightness}, getting assigned {ascii_symbols[level]}.")
            ascii_art += ascii_symbols[level]

        ascii_art += "\n"

    logger.info("Conversion successful")

    return ascii_art